@@ -8,8 +8,8 @@ from logging import Logger
 import time
 from typing import (
     Any,
+    cast,
     Dict,
-    Iterable,
     List,
     Mapping,
     Optional,
@@ -41,6 +41,55 @@ from .db import (
 )
 
 
+class MetricsLastSeen:
+    """Track last seen times for metrics.
+
+    It assumes labels are sorted by name in metrics.
+
+    """
+
+    def __init__(self, expirations: Dict[str, Optional[int]]):
+        self._expirations = expirations
+        self._last_seen: Dict[str, Dict[Tuple[str, ...], float]] = defaultdict(dict)
+
+    def update(
+        self,
+        name: str,
+        labels: Dict[str, str],
+        timestamp: float,
+    ):
+        """Update last seen for a metric with a set of labels to given timestamp."""
+        if not self._expirations.get(name):
+            return
+
+        # sort by label name
+        label_values = tuple(value for _, value in sorted(labels.items()))
+        self._last_seen[name][label_values] = timestamp
+
+    def expire_series(self, timestamp: float) -> Dict[str, List[Tuple[str, ...]]]:
+        """Expire and return expired metric series at the given timestamp.
+
+        Expired series are removed internally.
+
+        Return a dict mapping metric names to a list of tuples of sorted label
+        values for expired series.
+
+        """
+        expired = {}
+        for name, metric_last_seen in self._last_seen.items():
+            expiration = cast(int, self._expirations[name])
+            expired[name] = [
+                label_values
+                for label_values, last_seen in metric_last_seen.items()
+                if timestamp > last_seen + expiration
+            ]
+        # clear expired series from tracking
+        for name, series_labels in expired.items():
+            for label_values in series_labels:
+                del self._last_seen[name][label_values]
+        return expired
+
+
 class QueryLoop:
     """Run database queries and collect metrics."""
 
@@ -68,10 +117,22 @@ class QueryLoop:
         # map query names to list of database names
         self._doomed_queries: Dict[str, Set[str]] = defaultdict(set)
         self._loop = asyncio.get_event_loop()
-        # track last seen time for label values for metrics that have an
-        # expiration. The key is a tuple with metric name and label values
-        self._series_last_seen: Dict[Tuple, float] = {}
-        self._setup()
+        self._last_seen = MetricsLastSeen(
+            {
+                name: metric.config.get("expiration")
+                for name, metric in self._config.metrics.items()
+            }
+        )
+        self._databases: Dict[str, DataBase] = {
+            db_config.name: DataBase(db_config, logger=self._logger)
+            for db_config in self._config.databases.values()
+        }
+
+        for query in self._config.queries.values():
+            if query.timed:
+                self._timed_queries.append(query)
+            else:
+                self._aperiodic_queries.append(query)
 
     async def start(self):
         """Start timed queries execution."""
@@ -89,25 +150,16 @@ class QueryLoop:
         coros = (call.stop() for call in self._timed_calls.values())
         await asyncio.gather(*coros, return_exceptions=True)
         self._timed_calls.clear()
-        coros = (db.close() for db in self._databases)
+        coros = (db.close() for db in self._databases.values())
         await asyncio.gather(*coros, return_exceptions=True)
 
     def clear_expired_series(self):
         """Clear metric series that have expired."""
-        expired = []
-        now = self._timestamp()
-        for key, timestamp in self._series_last_seen.items():
-            name, *label_values = key
-            expiration = self._config.metrics[name].config["expiration"]
-            if timestamp + expiration > now:
-                continue
-
-            expired.append(key)
+        expired_series = self._last_seen.expire_series(self._timestamp())
+        for name, label_values in expired_series.items():
             metric = self._registry.get_metric(name)
-            metric.remove(*label_values)
-
-        for key in expired:
-            del self._series_last_seen[key]
+            for values in label_values:
+                metric.remove(*values)
 
     async def run_aperiodic_queries(self):
         """Run queries on request."""
@@ -127,22 +179,6 @@ class QueryLoop:
             delta = cc - t
             yield self._loop.time() + delta
 
-    @property
-    def _databases(self) -> Iterable[DataBase]:
-        """Return an iterable with defined Databases."""
-        return self._config.databases.values()
-
-    def _setup(self):
-        """Initialize instance attributes."""
-        for database in self._databases:
-            database.set_logger(self._logger)
-
-        for query in self._config.queries.values():
-            if query.timed:
-                self._timed_queries.append(query)
-            else:
-                self._aperiodic_queries.append(query)
-
     def _run_query(self, query: Query):
         """Periodic task to run a query."""
         for dbname in query.databases:
@@ -153,7 +189,7 @@ class QueryLoop:
         if await self._remove_if_dooomed(query, dbname):
             return
 
-        db = self._config.databases[dbname]
+        db = self._databases[dbname]
         try:
             metric_results = await db.execute(query)
         except DataBaseConnectError:
@@ -210,8 +246,8 @@ class QueryLoop:
         elif isinstance(value, Decimal):
             value = float(value)
         method = self._METRIC_METHODS[self._config.metrics[name].type]
-        all_labels = {DATABASE_LABEL: database.name}
-        all_labels.update(database.labels)
+        all_labels = {DATABASE_LABEL: database.config.name}
+        all_labels.update(database.config.labels)
         if labels:
             all_labels.update(labels)
         labels_string = ",".join(
@@ -222,15 +258,7 @@ class QueryLoop:
         )
         metric = self._registry.get_metric(name, labels=all_labels)
         getattr(metric, method)(value)
-        self._update_metric_expiration(name, all_labels)
-
-    def _update_metric_expiration(self, name: str, labels: Dict[str, str]):
-        if not self._config.metrics[name].config.get("expiration"):
-            return
-
-        # key by metric name and label values, sorted by label name
-        key = (name, *(value for _, value in sorted(labels.items())))
-        self._series_last_seen[key] = self._timestamp()
+        self._last_seen.update(name, all_labels, self._timestamp())
 
     def _increment_queries_count(self, database: DataBase, query: Query, status: str):
         """Increment count of queries in a status for a database."""
