@@ -4,6 +4,7 @@ import pytest
 import yaml
 
 from ..config import (
+    _get_parameters_sets,
     _resolve_dsn,
     ConfigError,
     DB_ERRORS_METRIC_NAME,
@@ -112,6 +113,20 @@ CONFIG_INVALID_METRICS_PARAMS_DIFFERENT_KEYS = {
     },
 }
 
+CONFIG_INVALID_METRICS_PARAMS_MATRIX_DIFFERENT_KEYS = {
+    "databases": {"db": {"dsn": "sqlite://"}},
+    "metrics": {"m": {"type": "gauge"}},
+    "queries": {
+        "q": {
+            "interval": 10,
+            "databases": ["db"],
+            "metrics": ["m"],
+            "sql": "SELECT :param AS m",
+            "parameters": {"a": [{"foo": 1}, {"bar": 2}]},
+        },
+    },
+}
+
 
 class TestLoadConfig:
     def test_load_databases_section(self, logger, write_config):
@@ -138,12 +153,10 @@ class TestLoadConfig:
         assert database1.dsn == "sqlite:///foo"
         assert database1.keep_connected
         assert database1.autocommit
-        assert database1._engine._execution_options == {"autocommit": True}
         assert database2.name == "db2"
         assert database2.dsn == "sqlite:///bar"
         assert not database2.keep_connected
         assert not database2.autocommit
-        assert database2._engine._execution_options == {"autocommit": False}
 
     def test_load_databases_dsn_from_env(self, logger, write_config):
         """The database DSN can be loaded from env."""
@@ -271,6 +284,23 @@ class TestLoadConfig:
             == 'Unable to read dsn file : "/not/found": No such file or directory'
         )
 
+    def test_load_databases_no_labels(self, logger, write_config):
+        """If no labels are defined, an empty dict is returned."""
+        config = {
+            "databases": {
+                "db": {
+                    "dsn": "sqlite://",
+                }
+            },
+            "metrics": {},
+            "queries": {},
+        }
+        config_file = write_config(config)
+        with config_file.open() as fd:
+            result = load_config(fd, logger)
+        db = result.databases["db"]
+        assert db.labels == {}
+
     def test_load_databases_labels(self, logger, write_config):
         """Labels can be defined for databases."""
         config = {
@@ -518,6 +548,65 @@ class TestLoadConfig:
             "param2": 20,
         }
 
+    def test_load_queries_section_with_parameters_matrix(self, logger, write_config):
+        """Queries can have parameters matrix."""
+        config = {
+            "databases": {"db": {"dsn": "sqlite://"}},
+            "metrics": {"m": {"type": "summary", "labels": ["l"]}},
+            "queries": {
+                "q": {
+                    "interval": 10,
+                    "databases": ["db"],
+                    "metrics": ["m"],
+                    "sql": "SELECT :marketplace__name AS l, :item__status AS m",
+                    "parameters": {
+                        "marketplace": [{"name": "amazon"}, {"name": "ebay"}],
+                        "item": [{"status": "active"}, {"status": "inactive"}],
+                    },
+                },
+            },
+        }
+        config_file = write_config(config)
+        with config_file.open() as fd:
+            result = load_config(fd, logger)
+
+        assert len(result.queries) == 4
+
+        # check common props for each query
+        for query_name, query in result.queries.items():
+            assert query.databases == ["db"]
+            assert query.metrics == [QueryMetric("m", ["l"])]
+            assert query.sql == "SELECT :marketplace__name AS l, :item__status AS m"
+
+        # Q1
+        query1 = result.queries["q[params0]"]
+        assert query1.name == "q[params0]"
+        assert query1.parameters == {
+            "marketplace__name": "amazon",
+            "item__status": "active",
+        }
+        # Q2
+        query2 = result.queries["q[params1]"]
+        assert query2.name == "q[params1]"
+        assert query2.parameters == {
+            "marketplace__name": "ebay",
+            "item__status": "active",
+        }
+        # Q3
+        query3 = result.queries["q[params2]"]
+        assert query3.name == "q[params2]"
+        assert query3.parameters == {
+            "marketplace__name": "amazon",
+            "item__status": "inactive",
+        }
+        # Q4
+        query4 = result.queries["q[params3]"]
+        assert query4.name == "q[params3]"
+        assert query4.parameters == {
+            "marketplace__name": "ebay",
+            "item__status": "inactive",
+        }
+
     def test_load_queries_section_with_wrong_parameters(self, logger, write_config):
         """An error is raised if query parameters don't match."""
         config = {
@@ -654,6 +743,11 @@ class TestLoadConfig:
                 'Invalid parameters definition for query "q": '
                 "parameters dictionaries must all have the same keys",
             ),
+            (
+                CONFIG_INVALID_METRICS_PARAMS_MATRIX_DIFFERENT_KEYS,
+                'Invalid parameters definition by path "a" for query "q": '
+                "parameters dictionaries must all have the same keys",
+            ),
         ],
     )
     def test_configuration_incorrect(self, logger, write_config, config, error_message):
@@ -844,3 +938,108 @@ class TestResolveDSN:
             _resolve_dsn(details, {})
             == "postgresql:///mydb?foo=a+value&bar=another%2Fvalue"
         )
+
+
+class TestGetParametersSets:
+    def test_list(self):
+        params = [
+            {
+                "param1": 100,
+                "param2": "foo",
+            },
+            {
+                "param1": 200,
+                "param2": "bar",
+            },
+        ]
+        assert list(_get_parameters_sets(params)) == params
+
+    def test_dict(self):
+        params = {
+            "param1": [
+                {
+                    "sub1": 100,
+                    "sub2": "foo",
+                },
+                {
+                    "sub1": 200,
+                    "sub2": "bar",
+                },
+            ],
+            "param2": [
+                {
+                    "sub3": "a",
+                    "sub4": False,
+                },
+                {
+                    "sub3": "b",
+                    "sub4": True,
+                },
+            ],
+            "param3": [
+                {
+                    "sub5": "X",
+                },
+                {
+                    "sub5": "Y",
+                },
+            ],
+        }
+        assert list(_get_parameters_sets(params)) == [
+            {
+                "param1__sub1": 100,
+                "param1__sub2": "foo",
+                "param2__sub3": "a",
+                "param2__sub4": False,
+                "param3__sub5": "X",
+            },
+            {
+                "param1__sub1": 100,
+                "param1__sub2": "foo",
+                "param2__sub3": "a",
+                "param2__sub4": False,
+                "param3__sub5": "Y",
+            },
+            {
+                "param1__sub1": 100,
+                "param1__sub2": "foo",
+                "param2__sub3": "b",
+                "param2__sub4": True,
+                "param3__sub5": "X",
+            },
+            {
+                "param1__sub1": 100,
+                "param1__sub2": "foo",
+                "param2__sub3": "b",
+                "param2__sub4": True,
+                "param3__sub5": "Y",
+            },
+            {
+                "param1__sub1": 200,
+                "param1__sub2": "bar",
+                "param2__sub3": "a",
+                "param2__sub4": False,
+                "param3__sub5": "X",
+            },
+            {
+                "param1__sub1": 200,
+                "param1__sub2": "bar",
+                "param2__sub3": "a",
+                "param2__sub4": False,
+                "param3__sub5": "Y",
+            },
+            {
+                "param1__sub1": 200,
+                "param1__sub2": "bar",
+                "param2__sub3": "b",
+                "param2__sub4": True,
+                "param3__sub5": "X",
+            },
+            {
+                "param1__sub1": 200,
+                "param1__sub2": "bar",
+                "param2__sub3": "b",
+                "param2__sub4": True,
+                "param3__sub5": "Y",
+            },
+        ]
