@@ -1,5 +1,6 @@
 import asyncio
 from collections import defaultdict
+from datetime import datetime
 from decimal import Decimal
 import logging
 from pathlib import Path
@@ -10,7 +11,10 @@ import pytest
 import yaml
 
 from .. import loop
-from ..config import load_config
+from ..config import (
+    DataBaseConfig,
+    load_config,
+)
 from ..db import DataBase
 
 
@@ -74,6 +78,15 @@ async def query_loop(make_query_loop):
     yield make_query_loop()
 
 
+@pytest.fixture
+def metrics_expiration():
+    yield {
+        "m1": 50,
+        "m2": 100,
+        "m3": None,
+    }
+
+
 def metric_values(metric, by_labels=()):
     """Return values for the metric."""
     if metric._type == "gauge":
@@ -82,7 +95,7 @@ def metric_values(metric, by_labels=()):
         suffix = "_total"
 
     values = defaultdict(list)
-    for sample_suffix, labels, value in metric._samples():
+    for sample_suffix, labels, value, *_ in metric._samples():
         if sample_suffix == suffix:
             if by_labels:
                 label_values = tuple(labels[label] for label in by_labels)
@@ -94,11 +107,46 @@ def metric_values(metric, by_labels=()):
 
 
 async def run_queries(db_file: Path, *queries: str):
-    async with DataBase("db", f"sqlite:///{db_file}") as db:
+    config = DataBaseConfig(name="db", dsn=f"sqlite:///{db_file}")
+    async with DataBase(config) as db:
         for query in queries:
             await db.execute_sql(query)
 
 
+class TestMetricsLastSeen:
+    def test_update(self, metrics_expiration):
+        """Last seen times are tracked for each series of metrics with expiration."""
+        last_seen = loop.MetricsLastSeen(metrics_expiration)
+        last_seen.update("m1", {"l1": "v1", "l2": "v2"}, 100)
+        last_seen.update("m1", {"l1": "v3", "l2": "v4"}, 200)
+        last_seen.update("other", {"l3": "v100"}, 300)
+        assert last_seen._last_seen == {
+            "m1": {
+                ("v1", "v2"): 100,
+                ("v3", "v4"): 200,
+            }
+        }
+
+    def test_update_label_values_sorted_by_name(self, metrics_expiration):
+        """Last values are sorted by label names."""
+        last_seen = loop.MetricsLastSeen(metrics_expiration)
+        last_seen.update("m1", {"l2": "v2", "l1": "v1"}, 100)
+        assert last_seen._last_seen == {"m1": {("v1", "v2"): 100}}
+
+    def test_expire_series(self, metrics_expiration):
+        """Expired metric series are returned and removed."""
+        last_seen = loop.MetricsLastSeen(metrics_expiration)
+        last_seen.update("m1", {"l1": "v1", "l2": "v2"}, 10)
+        last_seen.update("m1", {"l1": "v3", "l2": "v4"}, 100)
+        last_seen.update("m2", {"l3": "v100"}, 100)
+        expired = last_seen.expire_series(120)
+        assert expired == {"m1": [("v1", "v2")], "m2": []}
+        assert last_seen._last_seen == {
+            "m1": {("v3", "v4"): 100},
+            "m2": {("v100",): 100},
+        }
+
+
 @pytest.mark.asyncio
 class TestQueryLoop:
     async def test_start(self, query_loop):
@@ -206,7 +254,7 @@ class TestQueryLoop:
 
     async def test_update_metric_decimal_value(self, registry, make_query_loop):
         """A Decimal value in query results is converted to float."""
-        db = DataBase("db", "sqlite://")
+        db = DataBase(DataBaseConfig(name="db", dsn="sqlite://"))
         query_loop = make_query_loop()
         query_loop._update_metric(db, "m", Decimal("100.123"))
         metric = registry.get_metric("m")
@@ -265,7 +313,7 @@ class TestQueryLoop:
     ):
         """Count of database errors is incremented on failed connection."""
         query_loop = make_query_loop()
-        db = query_loop._config.databases["db"]
+        db = query_loop._databases["db"]
         mock_connect = mocker.patch.object(db._engine, "connect")
         mock_connect.side_effect = Exception("connection failed")
         await query_loop.start()
@@ -291,7 +339,7 @@ class TestQueryLoop:
         config_data["queries"]["q"]["timeout"] = 0.1
         query_loop = make_query_loop()
         await query_loop.start()
-        db = query_loop._config.databases["db"]
+        db = query_loop._databases["db"]
         await db.connect()
 
         async def execute(sql, parameters):
@@ -468,6 +516,8 @@ class TestQueryLoop:
             'INSERT INTO test VALUES (20, "bar")',
         )
         query_loop = make_query_loop()
+        mock_timestamp = mocker.patch.object(query_loop, "_timestamp")
+        mock_timestamp.return_value = datetime.now().timestamp()
         await query_loop.run_aperiodic_queries()
         await query_tracker.wait_results()
         queries_metric = registry.get_metric("m")
@@ -475,14 +525,12 @@ class TestQueryLoop:
             ("foo",): 10.0,
             ("bar",): 20.0,
         }
-        # mock that more time has passed than expiration
-        mocker.patch.object(query_loop, "_timestamp").return_value = (
-            query_loop._timestamp() + 20
-        )
         await run_queries(
             db,
             "DELETE FROM test WHERE m = 10",
         )
+        # mock that more time has passed than expiration
+        mock_timestamp.return_value += 20
         await query_loop.run_aperiodic_queries()
         await query_tracker.wait_results()
         query_loop.clear_expired_series()
